const dgram = require("dgram");
const radius = require('radius');
radius.add_dictionary(__ + '/dictionary');

global.Process = require(__ + '/modules/Process');

var server = dgram.createSocket("udp4");

server.on("message", function (msg, rinfo) {
  try {
    var packet = radius.decode({ packet: msg, secret: CONF.PROXY.SECRET });

    var mac_device = packet.attributes['Calling-Station-Id'];
    // var mac_ap = packet.attributes['Called-Station-Id'].substr(0, 17);
    var mac_ap = packet.attributes['Called-Station-Id'];

    if (!Util.isMAC(mac_device)) {
      var temp = mac_device.match(/(([0-9]|[A-Fa-f]){2})/g);
      mac_device = temp[0] + "-" + temp[1] + "-" + temp[2] + "-" + temp[3] + "-" + temp[4] + "-" + temp[5];
    }

    if (!Util.isMAC(mac_ap)) {
      var temp = mac_ap.match(/(([0-9]|[A-Fa-f]){2})/g);
      mac_ap = temp[0] + "-" + temp[1] + "-" + temp[2] + "-" + temp[3] + "-" + temp[4] + "-" + temp[5];
    }

    var nas_id = packet.attributes['NAS-Identifier'];   

    mac_ap = mac_ap.replace(/:/g, '-').replace(/("|')/g, "").toUpperCase();
    mac_device = mac_device.replace(/:/g, '-').replace(/("|')/g, "").toUpperCase();
    
    if (mac_ap == "00-00-00-00-00-00") {
      if (Util.isMAC(nas_id)) {
        mac_ap = nas_id = nas_id.replace(/:/g, '-').replace(/("|')/g, "").toUpperCase();
      }
    }

    logger.info("---END PROCESS---\n---START PROCESS---[" + mac_ap + "][" + mac_device + "]");
    logger.debug("[" + mac_ap + "][" + mac_device + "]:", "PACKET RECEIVED: ", JSON.stringify(packet))
    logger.debug("[" + mac_ap + "][" + mac_device + "]: ATTRIBUTES: ", JSON.stringify(packet.attributes));

    logger.info("[" + mac_ap + "][" + mac_device + "]:", "NAS Id: ", nas_id)

    if (packet.code != 'Access-Request') {
      logger.info('Invalid Packet: ', packet.code);
      return;
    }

    packet.secret = CONF.PROXY.SECRET;
    var obj = { packet: packet, secret: CONF.PROXY.SECRET, mac_ap: mac_ap, mac_device: mac_device, nas_id: nas_id }

    var start = (callback) => {
      logger.info("[" + mac_ap + "][" + mac_device + "]: [IP|PORT]", rinfo.address + ":" + rinfo.port);
      callback(null, obj);
    }
    var step1 = Process.getInfo; //Get profile by mac_device
    var step2 = Process.preprocess;

    var final = (response) => {
      server.send(response, 0, response.length, rinfo.port, rinfo.address, function (err, bytes) {
        if (err) {
          logger.debug("[" + mac_ap + "][" + mac_device + "]", 'Error sending response to ', rinfo);
        }
      });
    }

    async.waterfall([start, step1, step2], final);
  }
  catch (err) {
    logger.error(err);
  }
});

server.on("listening", function () {
  var address = server.address();
  logger.info("[RADIUS START] [PORT:" + address.port + "]");
});

server.bind(CONF.PORT);