global.mongoose = require('mongoose');
mongoose.Promise = global.Promise;

//Connect Mongo DB
// mongoose.connect(CONF.MONGO_URL, { useNewUrlParser: true });
var conn = mongoose.createConnection(CONF.MONGO_URL, { useNewUrlParser: true, poolSize: 2 });

//Handle events
conn.on('error', (err) => {
    logger.fatal(err);
    conn.disconnect();
}).on('connected', () => {
    logger.info('[MongoDB] connected ' + CONF.MONGO_URL)
}).on('reconnected', () => {
    logger.info('[MongoDB] reconnected')
}).on('disconnected', () => {
    conn = mongoose.createConnection(CONF.MONGO_URL, { useNewUrlParser: true, poolSize: 2 });
})

const SchemaObj = {
    Roaming: require(__ + '/models/schemas/Roaming'),
    TrustMac: require(__ + '/models/schemas/TrustMac'),
    TrustAp: require(__ + '/models/schemas/TrustAp'),
    Account: require(__ + '/models/schemas/Account'),
}

var createModelByName = (collection) => {
    var schemaName;

    if (collection.indexOf("_") > -1) {
        schemaName = collection.split("_")[1];
    }
    else {
        schemaName = collection;
    }

    if (schemaName in SchemaObj) {
        var schema = SchemaObj[schemaName];
        return conn.model(collection, schema, collection.toLowerCase());// ModelName, Schema, Collection
    }
    else {
        return false;
    }
}


module.exports = {
    find: (args, callback) => {
        var schema = args.schema;
        var query = args.query;
        var select = args.select;

        var model = createModelByName(schema)
        if (model) {

            var cmd = model.find(query)
            if (select) cmd.select(select);
            cmd.exec((err, docs) => {
                if (err) return callback(err)
                return callback(err, docs);
            })
        }
        else {
            callback("Schema [" + schema + "] Not Found", null)
        }
    },
    findLimit: (args, callback) => {
        var schema = args.schema;
        var query = args.query;
        var select = args.select;

        var model = createModelByName(schema)
        if (model) {
            model.count(query.search).exec((error, count) => {
                var cmd = model.find(query.search).skip(query.skip).limit(query.pageSize).sort(query.sort);
                if (select) cmd.select(select);

                cmd.exec((err, docs) => {
                    if (err) return callback(err)
                    return callback(err, docs);
                })
            })
        }
        else {
            callback("Schema [" + schema + "] Not Found", null)
        }
    },
    findOne: (args, callback) => {
        var schema = args.schema;
        var query = args.query;
        var select = args.select;

        var model = createModelByName(schema)
        if (model) {
            var cmd = model.findOne(query);
            if (select) cmd.select(select);

            cmd.exec((err, doc) => {
                if (err) return callback(err, null)
                return callback(err, doc);
            })
        }
        else {
            callback("Schema [" + schema + "] Not Found", null)
        }
    },
    findInCache: (args, callback) => {
        var schema = args.schema;
        var query = args.query;
        var select = args.select;

        Cache.get(schema, JSON.stringify(query), (err, docs) => {
            if (err) return callback(err, docs);
            if (docs) return callback(err, docs)
            var model = createModelByName(schema)
            if (model) {
                model.find(query || {})
                .select(select || "")
                .lean()
                .exec((err, docs) => {
                  if (err) return callback(err);
                  Cache.set(schema, JSON.stringify(query), docs, (err, docs) => {
                    if (err) console.error(err);
                  });
                  return callback(err, docs);
                })
            }
            else {
                callback("Schema [" + schema + "] Not Found", null)
            }
        })
    },
    findOneInCache: (args, callback) => {
        var schema = args.schema;
        var query = args.query;
        var select = args.select;

        Cache.get(schema, JSON.stringify(args), (err, docs) => {
            if (err) return callback(err, docs);
            if (docs) return callback(err, docs)
            var model = createModelByName(schema)
            if (model) {
                var cmd = model.findOne(query);
                if (select) cmd.select(select);

                cmd.exec((err, docs) => {
                    if (err) return callback(err, null)
                    Cache.set(schema, JSON.stringify(query), docs, (err, docs) => {
                        if (err) logger.error(err)
                    })
                    return callback(err, docs);
                })
            }
            else {
                callback("Schema [" + schema + "] Not Found", null)
            }
        })
    },
    findOneAndUpsert: (args, callback) => {
        var schema = args.schema;
        var query = args.query;
        var update = args.update;

        var model = createModelByName(schema)
        if (model) {
            var cmd = model.findOneAndUpdate(query, update, { upsert: true, new: true, returnNewDocument: true })
            cmd.exec((err, doc) => {
                callback(err, doc);
            })
        }
        else {
            callback("Schema [" + schema + "] Not Found", null)
        }
    },
    findOneAndUpdate: (args, callback) => {
        var schema = args.schema;
        var query = args.query;
        var update = args.update;

        var model = createModelByName(schema)
        if (model) {
            model.findOneAndUpdate(query, update, { upsert: false }, (err, doc) => {
                callback(err, doc);
            })
        }
        else {
            callback("Schema [" + schema + "] Not Found", null)
        }
    },
    findOneAndRemove: (args, callback) => {
        var schema = args.schema;
        var query = args.query;
        var select = args.select;

        var model = createModelByName(schema)
        if (model) {
            model.findOneAndRemove(query, (err) => {
                return callback(err, null);
            })
        }
        else {
            callback("Schema [" + schema + "] Not Found", null)
        }
    },
    //DELETE -
    findOneAndDelete: (args, callback) => {
        var schema = args.schema;
        var query = args.query;
        var select = args.select;

        var model = createModelByName(schema)
        if (model) {
            model.findOneAndDelete(query, { rawResult: true }, (err, result) => {
                return callback(err, result);
            });
        }
        else {
            return callback("Schema [" + schema + "] Not Found", null)
        }
    },
    populate: (args, callback) => {
        var schema = args.schema;
        var query = args.query;
        var select = args.select;
        var populate = args.populate;

        Cache.get(schema, JSON.stringify(query), (err, docs) => {
            if (err) return callback(err, docs);
            if (docs) return callback(err, docs);
            var model = createModelByName(schema)
            if (model) {
                var cmd = model.find(query).populate(populate)
                if (select) cmd.select(select);
                cmd.exec((err, docs) => {
                    if (err) return callback(err)
                    return callback(err, docs);
                })
            }
            else {
                callback("Schema [" + schema + "] Not Found", null)
            }
        });
    },
    populateInCache: (args, callback) => {
        var schema = args.schema;
        var query = args.query;
        var select = args.select;
        var populate = args.populate;

        Cache.get(schema, JSON.stringify(args), (err, docs) => {
            if (err) return callback(err, docs);
            if (docs) return callback(err, docs);
            var model = createModelByName(schema)
            if (model) {
                var cmd = model.find(query).populate(populate)
                if (select) cmd.select(select);
                cmd.exec((err, docs) => {
                    if (err) return callback(err)
                    if (docs) {
                        Cache.set(schema, JSON.stringify(args), docs, (err, docs) => {
                            if (err) logger.error(err)
                        })
                    }
                    return callback(err, docs);
                })
            }
            else {
                callback("Schema [" + schema + "] Not Found", null)
            }
        });
    },
    populateOne: (args, callback) => {
        var schema = args.schema;
        var query = args.query;
        var select = args.select;
        var populate = args.populate;

        var model = createModelByName(schema)
        if (model) {
            var cmd = model.find(query).populate(populate)
            if (select) cmd.select(select)

            cmd.exec((err, docs) => {
                if (err) return callback(err)
                return callback(err, docs);
            })
        }
        else {
            callback("Schema [" + schema + "] Not Found", null)
        }
    },
}