const async = require("async");
const Redis = require("redis").createClient(
    { port: CONF.REDIS.port, host: CONF.REDIS.host, password: CONF.REDIS.password }
).on('error', (err) => {
    logger.fatal('[REDIS] Error: ', err)
}).on('ready', () => {
    logger.debug('[REDIS] connected: ' + JSON.stringify(CONF.REDIS))
})

module.exports = {
    //Stringify JSON and set Cache
    set: (prefix, suffix, docs, callback) => {
        logger.debug(CONF.CACHE_PREFIX + prefix, suffix)
        var keycache = CONF.CACHE_PREFIX + prefix + ((typeof suffix != "undefined" && suffix != '') ? Util.hashKey(JSON.stringify(suffix)) : '');
        logger.debug("[set] [KeyCache]", keycache)
        Redis.set(keycache, JSON.stringify(docs), (err, result) => {
            return callback(err, result);
        })
    },
    //Get cache and parse JSON
    get: (prefix, suffix, callback) => {
        logger.debug(CONF.CACHE_PREFIX + prefix, suffix)
        suffix = ((typeof suffix != "undefined" && suffix != '') ? Util.hashKey(JSON.stringify(suffix)) : '')
        var keycache = CONF.CACHE_PREFIX + prefix + suffix;
        logger.debug("[get] [KeyCache]", keycache)
        Redis.get(keycache, (err, result) => {
            return callback(err, JSON.parse(result));
        })
    },
    getAll: (callback) => {
        Redis.keys(CONF.CACHE_PREFIX + "*", (err, result) => {
            logger.debug("Cache", result);
            callback(result)

        })
    },
    //Get cache and parse JSON
    getByPrefix: (prefix, callback) => {
        logger.debug(CONF.CACHE_PREFIX + prefix)
        var keycache = CONF.CACHE_PREFIX + prefix + '*';
        logger.debug("[get] [KeyCache]", keycache)
     
        Redis.keys(keycache, (err, keys) => {
            if (err) return callback(err, keys);
            logger.debug("[KeyCache]", Object.keys(keys));                
            return callback(err, (keys));
        })
    },
    clearByPrefix: (prefix, suffix, callback) => {
        var suffix = ((typeof suffix != "undefined" && suffix != '') ? Util.hashKey(JSON.stringify(suffix)) : '*')
        var keycache = CONF.CACHE_PREFIX + prefix + suffix;
        logger.debug("[remove] [KeyCache]", keycache);

        if(suffix == "*") { // * is all
            Redis.keys(keycache, (err, keys) => {
                if (err) return callback(err, keys);
                logger.debug("[KeyCache]", keys);
                keys.forEach(key => {
                    logger.debug("[clear] [KeyCache]", keycache);
                    Redis.del(key, (err, result) => { })
                })
                return callback(err, {});
            })
        }
        else {
            Redis.del(keycache, (err, result) => {
                return callback(err, JSON.parse(result));
            })
        }
    },
    //Clear all cache by prefix
    clear: (prefix, callback) => {
        var keycache = CONF.CACHE_PREFIX + prefix + "*";
        Redis.keys(keycache, (err, keys) => {
            if (err) return callback(err, keys);
            logger.debug("[KeyCache]", keys);
            keys.forEach(key => {
                logger.debug("[clear] [KeyCache]", key);
                Redis.del(key, (err, result) => { })
            })
            return callback(err, {});
        })
    },
    clearAll: (callback) => {
        var keycache = CONF.CACHE_PREFIX + "*";
        Redis.keys(keycache, (err, keys) => {
            if (err) return callback(err, keys);
            logger.debug("[clearAll] [KeyCache]", keys);
            keys.forEach(key => {
                Redis.del(key, (err, result) => { })
            })
            return callback(null, {});
        })
    },
    //Clear all cache by prefix
    clearSession: (mac, callback) => {
        var keycache = "RADIUS_MacAuth" + mac + "*";
        logger.debug(keycache);
        Redis.keys(keycache, (err, keys) => {
            if (err) return callback(err, keys);
            logger.debug("[KeyCache]", keys);
            keys.forEach(key => {
                logger.debug("[clear] [KeyCache]", keycache);
                Redis.del(key, (err, result) => { });
            })
            return callback(err, {});
        })
    },
}