const radius = require('radius');
radius.add_dictionary(__ + '/dictionary');

module.exports = {
    getInfo: (obj, callback) => {
        var mac_device = obj.mac_device;
        var mac_ap = obj.mac_ap;

        obj.args = {};
        obj.args.username = obj.packet.attributes['User-Name'];
        obj.args.password = obj.packet.attributes['User-Password'];

        if (obj.args.password == null) {
            obj.args.password = obj.args.username;
        }
        else if (/�/g.test(obj.args.password)) { //Accept [0-9][A-Za-z] '/' '_' '-'
            obj.args.password = obj.args.username;
        }

        logger.debug("[" + mac_ap + "][" + mac_device + "]:", '[GetInfo]: Username/Password = ' + obj.args.username + "/" + obj.args.password);

        callback(null, obj);
    },
    preprocess: (obj, callback) => {
        var mac_device = obj.mac_device;
        var mac_ap = obj.mac_ap;

        var username = obj.args.username;
        if (Util.isMAC(username) || Util.isMACNoDelimiter(username)) {
            logger.info("[" + mac_ap + "][" + mac_device + "]:", "[BYMAC]", obj.args.username, obj.args.password);
            Process.authByMAC(obj, (err, obj) => {
                return callback(err, obj);
            })
        }
        else {
            logger.info("[" + mac_ap + "][" + mac_device + "]:", "[BYPASS]", obj.args.username, obj.args.password);
            Process.authByPass(obj, (err, res) => {
                return callback(err, obj);
            })
        }
    },
    authByMAC: (obj, callback) => {
        var mac_device = obj.mac_device;
        var mac_ap = obj.mac_ap;
        var nas_id = obj.nas_id;

        var AcctSessionId = obj.packet.attributes['Acct-Session-Id'];
        var CalledStationId = obj.packet.attributes['Called-Station-Id'];

        var $in = [mac_ap];

        if (Util.isMAC(nas_id)) {
            $in.push(nas_id);
        }

        //Check TrustMac
        var query_args = { schema: "TrustMac", query: { mac_device: mac_device, status: "Active", $or: [{ expired: { $gt: Util.now() } }, { expired: null }] } };
        Radius.findOne(query_args, (err, doc) => {
            if (err) {
                logger.error("[" + mac_ap + "][" + mac_device + "]:", err)
            }

            var packet = {};
            packet.packet = obj.packet;
            packet.secret = obj.secret;

            if (doc) {
                logger.info("[" + mac_ap + "][" + mac_device + "]:", "[TRUSTMAC]");
                doc = JSON.parse(JSON.stringify(doc));
                // logger.info(doc);
                var diff = 1;

                if (typeof doc.expired != "undefined") {
                    diff = moment(doc.expired).diff(moment().utcOffset(420), "seconds");
                    logger.info("[" + mac_ap + "][" + mac_device + "]:", '[TRUSTMAC] [EXPIRY DATE]:', doc.expired, "REMAIN: ", diff);
                }

                if (diff > 0) {
                    doc.bwup = (doc.bwup == -1) ? 1000000000 : doc.bwup;
                    doc.bwdown = (doc.bwdown == -1) ? 1000000000 : doc.bwdown;

                    packet.code = 'Access-Accept';
                    packet.add_message_authenticator = true;
                    packet.attributes = [];
                    packet.attributes.push(['Vendor-Specific', 14122, [['WISPr-Bandwidth-Max-Up', doc.bwup], ['WISPr-Bandwidth-Max-Down', doc.bwdown]]])
                    packet.attributes.push(['Vendor-Specific', 14559, [['CoovaChilli-Bandwidth-Max-Up', doc.bwup], ['CoovaChilli-Bandwidth-Max-Down', doc.bwdown]]])

                    packet.attributes.push(['Vendor-Specific', 14179, [['Airespace-Data-Bandwidth-Average-Contract-Upstream', doc.bwup / 1000],
                    ['Airespace-Real-Time-Bandwidth-Average-Contract-Upstream', doc.bwup / 1000],
                    ['Airespace-Data-Bandwidth-Burst-Contract-Upstream', doc.bwup / 1000],
                    ['Airespace-Real-Time-Bandwidth-Burst-Contract-Upstream', doc.bwup / 1000],
                    ['Airespace-Data-Bandwidth-Average-Contract', doc.bwdown / 1000],
                    ['Airespace-Real-Time-Bandwidth-Average-Contract', doc.bwdown / 1000],
                    ['Airespace-Data-Bandwidth-Burst-Contract', doc.bwdown / 1000],
                    ['Airespace-Real-Time-Bandwidth-Burst-Contract', doc.bwdown / 1000]]])


                    // packet.attributes.push(['Vendor-Specific', 17713, [['CAMB-Traffic-Quota-Limit-Up', doc.bwup / 1000], ['CAMB-Traffic-Quota-Limit-Down', doc.bwdown / 1000]]])

                    packet.attributes.push(['Session-Timeout', doc.session])
                    packet.attributes.push(['Idle-Timeout', doc.idle])

                    // if ("Message-Authenticator" in obj.packet.attributes) {
                    //     packet.attributes.push(["Message-Authenticator", obj.packet.attributes["Message-Authenticator"]])
                    // }

                    var response = radius.encode_response(packet);
                    logger.info("[" + mac_ap + "][" + mac_device + "]:", "[TRUSTMAC] [ACCEPT]");

                    return callback(response);
                }
            }

            //Check TrustAp
            var query_args = { schema: "TrustAp", query: { mac_ap: mac_ap, status: "Active" } };
            Radius.findOne(query_args, (err, doc) => {
                if (err) {
                    logger.error("[" + mac_ap + "][" + mac_device + "]:", err)
                }

                if (doc) {
                    doc.bwup = (doc.bwup == -1) ? 1000000000 : doc.bwup;
                    doc.bwdown = (doc.bwdown == -1) ? 1000000000 : doc.bwdown;

                    logger.info("[" + mac_ap + "][" + mac_device + "]:", "[TRUSTAP]");
                    doc = JSON.parse(JSON.stringify(doc));

                    var packet = {}
                    packet.code = 'Access-Accept';
                    packet.packet = obj.packet;
                    packet.secret = obj.secret;
                    packet.add_message_authenticator = true;
                    packet.attributes = [];
                    packet.attributes.push(['Vendor-Specific', 14122, [['WISPr-Bandwidth-Max-Up', doc.bwup], ['WISPr-Bandwidth-Max-Down', doc.bwdown]]])
                    packet.attributes.push(['Vendor-Specific', 14559, [['CoovaChilli-Bandwidth-Max-Up', doc.bwup], ['CoovaChilli-Bandwidth-Max-Down', doc.bwdown]]])

                    packet.attributes.push(['Vendor-Specific', 14179, [['Airespace-Data-Bandwidth-Average-Contract-Upstream', doc.bwup / 1000],
                    ['Airespace-Real-Time-Bandwidth-Average-Contract-Upstream', doc.bwup / 1000],
                    ['Airespace-Data-Bandwidth-Burst-Contract-Upstream', doc.bwup / 1000],
                    ['Airespace-Real-Time-Bandwidth-Burst-Contract-Upstream', doc.bwup / 1000],
                    ['Airespace-Data-Bandwidth-Average-Contract', doc.bwdown / 1000],
                    ['Airespace-Real-Time-Bandwidth-Average-Contract', doc.bwdown / 1000],
                    ['Airespace-Data-Bandwidth-Burst-Contract', doc.bwdown / 1000],
                    ['Airespace-Real-Time-Bandwidth-Burst-Contract', doc.bwdown / 1000]]])


                    // packet.attributes.push(['Vendor-Specific', 17713, [['CAMB-Traffic-Quota-Limit-Up', doc.bwup / 1000], ['CAMB-Traffic-Quota-Limit-Down', doc.bwdown / 1000]]])


                    packet.attributes.push(['Session-Timeout', doc.session])
                    packet.attributes.push(['Idle-Timeout', doc.idle])

                    // if ("Message-Authenticator" in obj.packet.attributes) {
                    //     packet.attributes.push(["Message-Authenticator", obj.packet.attributes["Message-Authenticator"]])
                    // }

                    var response = radius.encode_response(packet);
                    logger.info("[" + mac_ap + "][" + mac_device + "]:", '[TRUSTAP] [ACCEPT]');

                    return callback(response);
                }

                var query_args = { schema: "Roaming", query: { mac_ap: { $in: $in }, status: "Active" } }

                Radius.findOne(query_args, (err, doc) => {
                    var prefix = "MacAuth";
                    var keycache = mac_device;

                    if (doc) {
                        keycache += doc._id;
                    }
                    else {
                        if (mac_ap == "00-00-00-00-00-00" && Util.isMAC(nas_id)) {
                            keycache += nas_id;
                        }
                        else {
                            keycache += mac_ap;
                        }
                    }

                    Cache.get(prefix + keycache, "", (err, data) => {
                        if (err) logger.error(err)

                        logger.info("[" + mac_ap + "][" + mac_device + "]:", "[ROAMING] [RADIUS_" + prefix + keycache + "]: ", JSON.stringify(data));

                        try {
                            if (data != null) {
                                logger.info("[" + mac_ap + "][" + mac_device + "]:", "[ROAMING] [EXPIRED]", moment(data.expired).utcOffset(420).format("YYYY-MM-DD HH:mm:ss"))
                                var diff = moment(data.expired).diff(moment().utcOffset(420), "seconds");
                                logger.info("[" + mac_ap + "][" + mac_device + "]:", "[ROAMING] [REMAINING]: ", diff + "s")

                                if (diff > 0) {
                                    data.session_timeout = diff;
                                    var packet = {}
                                    packet.code = 'Access-Accept';
                                    packet.packet = obj.packet;
                                    packet.secret = obj.secret;
                                    packet.add_message_authenticator = true;
                                    packet.attributes = [];
                                    // if (data.bwup <= 0 && data.bwdown <= 0) {
                                    packet.attributes.push(['Vendor-Specific', 14122, [['WISPr-Bandwidth-Max-Up', data.bwup], ['WISPr-Bandwidth-Max-Down', data.bwdown]]])
                                    packet.attributes.push(['Vendor-Specific', 14559, [['CoovaChilli-Bandwidth-Max-Up', data.bwup], ['CoovaChilli-Bandwidth-Max-Down', data.bwdown]]])

                                    packet.attributes.push(['Vendor-Specific', 14179, [['Airespace-Data-Bandwidth-Average-Contract-Upstream', data.bwup / 1000],
                                    ['Airespace-Real-Time-Bandwidth-Average-Contract-Upstream', data.bwup / 1000],
                                    ['Airespace-Data-Bandwidth-Burst-Contract-Upstream', data.bwup / 1000],
                                    ['Airespace-Real-Time-Bandwidth-Burst-Contract-Upstream', data.bwup / 1000],
                                    ['Airespace-Data-Bandwidth-Average-Contract', data.bwdown / 1000],
                                    ['Airespace-Real-Time-Bandwidth-Average-Contract', data.bwdown / 1000],
                                    ['Airespace-Data-Bandwidth-Burst-Contract', data.bwdown / 1000],
                                    ['Airespace-Real-Time-Bandwidth-Burst-Contract', data.bwdown / 1000]]])


                                    // packet.attributes.push(['Vendor-Specific', 17713, [['CAMB-Traffic-Quota-Limit-Up', data.bwup / 1000], ['CAMB-Traffic-Quota-Limit-Down', data.bwdown / 1000]]])


                                    packet.attributes.push(['Session-Timeout', data.session_timeout])
                                    packet.attributes.push(['Idle-Timeout', data.idle_timeout || 60])

                                    // if ("Message-Authenticator" in obj.packet.attributes) {
                                    //     packet.attributes.push(["Message-Authenticator", obj.packet.attributes["Message-Authenticator"]])
                                    // }

                                    var response = radius.encode_response(packet);

                                    logger.info("[" + mac_ap + "][" + mac_device + "]:", '[ROAMING] [ACCEPT]');

                                    return callback(response);
                                }
                                else {
                                    logger.info("[" + mac_ap + "][" + mac_device + "]:", '[ROAMING] [EXPIRED]');
                                    Cache.clearByPrefix(prefix + keycache, "", (err, data) => {
                                        logger.info("[" + mac_ap + "][" + mac_device + "]: [ROAMING] [CACHE REMOVED]");
                                    })
                                }
                            }

                            var packet = {}
                            packet.code = 'Access-Reject';
                            packet.packet = obj.packet;
                            packet.secret = obj.secret;

                            var response = radius.encode_response(packet);
                            logger.info("[" + mac_ap + "][" + mac_device + "]:", '[RESPONSE]: [REJECT]');

                            return callback(response);
                        }
                        catch (err) {
                            logger.error(err);
                            var packet = {}
                            packet.code = 'Access-Reject';
                            packet.packet = obj.packet;
                            packet.secret = obj.secret;

                            var response = radius.encode_response(packet);
                            logger.info("[" + mac_ap + "][" + mac_device + "]:", '[RESPONSE]: [REJECT]');

                            return callback(response);
                        }
                    });
                })
            })

        });
    },
    authByPass: (obj, callback) => {
        var mac_device = obj.mac_device;
        var mac_ap = obj.mac_ap;

        var query_args = { schema: 'Account', query: { username: obj.args.username, password: obj.args.password } };
        Radius.findOne(query_args, (err, account) => {
            if (err) {
                logger.error(err)
            }
            else if (account) {
                obj.code = 'Access-Accept';

                account = JSON.parse(JSON.stringify(account));

                var packet = {}
                packet.code = obj.code;
                packet.packet = obj.packet;
                packet.secret = obj.secret;
                packet.attributes = [
                    ['Vendor-Specific', 14122, [['WISPr-Bandwidth-Max-Up', account.bwup], ['WISPr-Bandwidth-Max-Down', account.bwdown]]],
                    ['Vendor-Specific', 14559, [['CoovaChilli-Bandwidth-Max-Up', account.bwup], ['CoovaChilli-Bandwidth-Max-Down', account.bwdown]]],
                    ['Vendor-Specific', 14179, [['Airespace-Data-Bandwidth-Average-Contract-Upstream', account.bwup / 1000],
                    ['Airespace-Real-Time-Bandwidth-Average-Contract-Upstream', account.bwup / 1000],
                    ['Airespace-Data-Bandwidth-Burst-Contract-Upstream', account.bwup / 1000],
                    ['Airespace-Real-Time-Bandwidth-Burst-Contract-Upstream', account.bwup / 1000],
                    ['Airespace-Data-Bandwidth-Average-Contract', account.bwdown / 1000],
                    ['Airespace-Real-Time-Bandwidth-Average-Contract', account.bwdown / 1000],
                    ['Airespace-Data-Bandwidth-Burst-Contract', account.bwdown / 1000],
                    ['Airespace-Real-Time-Bandwidth-Burst-Contract', account.bwdown / 1000]]],

                    ['Vendor-Specific', 17713, [['CAMB-Traffic-Quota-Limit-Up', account.bwup / 1000], ['CAMB-Traffic-Quota-Limit-Down', account.bwdown / 1000]]],

                    ['Session-Timeout', account.session],
                    ['Idle-Timeout', account.idle]
                ];

                // ['Airespace-Data-Bandwidth-Average-Contract-Downstream', account.bwdown],
                // ['Airespace-Real-Time-Bandwidth-Average-Contract-Downstream', account.bwdown],
                // ['Airespace-Data-Bandwidth-Burst-Contract-Downstream ', account.bwdown],
                // ['Airespace-Real-Time-Bandwidth-Burst-Contract-Downstream ', account.bwdown],
                logger.info("[" + mac_ap + "][" + mac_device + "]:", '[BYPASS] [ACCEPT]', '["' + obj.args.username + '", "' + obj.args.password + '"]');

                var response = radius.encode_response(packet);
                var query_args = { schema: "Roaming", query: { mac_ap: mac_ap, status: "Active" } }
                Radius.findOne(query_args, (err, doc) => {
                    if (err) logger.error(err);

                    var prefix = "MacAuth";
                    var keycache = mac_device;
                    var session_timeout = account.session;
                    var idle_timeout = account.idle;

                    if (doc) {
                        keycache += doc._id;
                    }
                    else {
                        keycache += mac_ap;
                    }

                    var expired = moment().utcOffset(420).add(session_timeout, 'seconds').format("YYYY-MM-DD HH:mm:ss");

                    var data = { bwup: account.bwup, bwdown: account.bwdown, idle_timeout: idle_timeout, expired: expired };

                    Cache.set(prefix + keycache, "", data, (err) => {
                        logger.info("[" + mac_ap + "][" + mac_device + "]:", "[BYPASS] [ROAMING CACHED]", "[RADIUS_" + prefix + keycache + "]: ", JSON.stringify(data));
                    });
                })

                return callback(response);
            }

            logger.info("[" + mac_ap + "][" + mac_device + "]:", '[BYPASS]: [REJECT] Invalid username or password!')
            var packet = {}
            packet.code = 'Access-Reject';
            packet.packet = obj.packet;
            packet.secret = obj.secret;

            var response = radius.encode_response(packet);

            return callback(response);
        })
    },
}